@@ -20,7 +20,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 import argparse
 import datetime
 import json
-import logging
+import logging.handlers
 import os
 import re
 from dataclasses import asdict
@@ -89,7 +89,6 @@ class Configuration:
     youtube_cookie_file: Optional[str]
 
     def __init__(self: CFG, config_file_name: str) -> None:
-
         if os.path.exists(config_file_name):
             with open(file=config_file_name, encoding="UTF-8") as config_file:
                 config = json.load(config_file)
@@ -213,7 +212,7 @@ class Configuration:
 
 
 def main() -> None:
-    """Main / Overall Logic of Youtube Playlist to podcast feed conversion.
+    """Main / Overall Logic of YouTube Playlist to podcast feed conversion.
 
     :param: None
     :return: None
@@ -224,6 +223,21 @@ def main() -> None:
     parser.add_argument(
         "-c", "--config-file", action="store", default="config.json", dest="config_file"
     )
+    parser.add_argument(
+        "-l",
+        "--debug-log-file",
+        action="store",
+        dest="debug_log_file",
+        help="Path of filename to save DEBUG log messages to",
+    )
+    parser.add_argument(
+        "-p",
+        "--publish-dir",
+        action="store",
+        dest="publish_dir",
+        help="Directory to save feed.rss and media to. Overrides setting config file",
+    )
+
     args = parser.parse_args()
 
     config = Configuration(config_file_name=args.config_file)
@@ -239,10 +253,28 @@ def main() -> None:
     std_out_handler.setLevel(config.log_level)
     logger.addHandler(std_out_handler)
 
+    if args.debug_log_file:
+        file_log_formatter = logging.Formatter(
+            "%(asctime)s %(name)s[%(process)d] %(levelname)s %(message)s",
+            datefmt="%H:%M:%S",
+        )
+        file_handler = logging.handlers.TimedRotatingFileHandler(
+            filename=args.debug_log_file,
+            backupCount=7,
+            when="midnight",
+        )
+        file_handler.setFormatter(file_log_formatter)
+        file_handler.setLevel(logging.DEBUG)
+        logger.addHandler(file_handler)
+
+    if args.publish_dir:
+        config.publish_dir = args.publish_dir
+
     rprint(
         f"[cyan]Welcome to [green][bold]{__display_name__}[/bold][cyan] "
         f"version [yellow]{__version__}"
     )
+    logger.debug("Welcome to %s version %s", __display_name__, __version__)
 
     check_updates()
 
@@ -253,18 +285,26 @@ def main() -> None:
     feed.title("Marvin's Youtube Playlist Podcast")
     feed.description("Marvin's Youtube Playlist Podcast")
 
+    download_options = YDL_DL_OPTS
+    download_options[
+        "outtmpl"
+    ] = f"{config.publish_dir}/{config.media_dir}/%(id)s.%(ext)s"
+    download_options["playlistend"] = config.number_of_episodes * 3
+    if config.youtube_cookie_file and config.youtube_cookie_file != "":
+        download_options["cookiefile"] = config.youtube_cookie_file
+
     for current_play_list in config.play_lists:
         rprint(f"Downloading info for videos on playlist: {current_play_list['url']}")
-
-        download_options = YDL_DL_OPTS
-        download_options["playlistend"] = config.number_of_episodes * 3
-        if config.youtube_cookie_file and config.youtube_cookie_file != "":
-            download_options["cookiefile"] = config.youtube_cookie_file
+        logger.debug(
+            "Downloading info for videos on playlist: %s", current_play_list["url"]
+        )
         downloader = YoutubeDL(download_options)
-
-        videos_list = downloader.extract_info(current_play_list["url"], download=False)[
-            "entries"
-        ]
+        download_info = downloader.extract_info(
+            url=current_play_list["url"],
+            download=False,
+        )
+        logger.debug("main() - download_info = %s", json.dumps(download_info, indent=4))
+        videos_list = download_info["entries"]
 
         videos_to_download = [
             video
@@ -301,7 +341,7 @@ def is_video_included(
     :param video_original_url: Original URL of video to check filters against.
     :param include_filters: List of regex patterns acting as include filters.
         If one include filter matches, video will be considered for adding.
-    :param exclude_filters: Lost of regex patterns actins as exclude filters.
+    :param exclude_filters: List of regex patterns acting as exclude filters.
         If one exclude filter matches, video will be skipped, even if there is a
         matching include filter. I.e. exclude > include
     :return: True if video should be considered for adding and False if video should
@@ -378,11 +418,11 @@ def add_episodes(
             feed_entry = feed.add_entry()
 
             feed_entry.author(name=video["uploader"])
-            feed_entry.id(video["id"])
+            feed_entry.id(id=video["id"])
             feed_entry.link(href=video["webpage_url"])
-            feed_entry.title(video["title"])
-            feed_entry.description(video["description"])
-            feed_entry.published(published_on.datetime)
+            feed_entry.title(title=video["title"])
+            feed_entry.description(description=video["description"])
+            feed_entry.published(published=published_on.datetime)
 
             # Download video if needed
             local_audio_file = (
@@ -398,10 +438,21 @@ def add_episodes(
                     f"uploaded on {feed_entry.published().strftime('%Y-%m-%d')} with "
                     f"title [green]{feed_entry.title()}"
                 )
-                downloader.download([video["webpage_url"]])
-            feed_entry.enclosure(host_audio_file, 0, "audio/ogg")
+                logger.debug(
+                    "Downloading episode with id %s and length %s uploaded on %s with title %s",
+                    feed_entry.id(),
+                    datetime.timedelta(seconds=int(video["duration"])),
+                    feed_entry.published().strftime("%Y-%m-%d"),
+                    feed_entry.title(),
+                )
+                downloader.download(url_list=[video["webpage_url"]])
+            feed_entry.enclosure(url=host_audio_file, type="audio/ogg")
 
-            thumbnail = get_thumbnail(config, feed_entry.id(), video)
+            thumbnail = get_thumbnail(
+                config=config,
+                video_id=feed_entry.id(),
+                video=video,
+            )
             # pylint: disable=no-member
             feed_entry.podcast.itunes_image(
                 f"{config.podcast_host}/{config.media_dir}/{thumbnail}"
@@ -461,10 +512,16 @@ def check_updates() -> None:
                 f"[bold][red]!!! New version of {__display_name__} ({pypi_version}) "
                 f"is available on PyPI.org !!!\n"
             )
+            logger.debug(
+                "!!! New version of %s (%s) is available on PyPI.org",
+                __display_name__,
+                pypi_version,
+            )
     except ValueError:
         rprint(
             "[yellow]Notice - Your version is higher than last published version on PyPI"
         )
+        logger.debug("Notice - Your version is higher than the last publish version")
 
 
 def get_thumbnail(config: Configuration, video_id: str, video: Dict[str, Any]) -> str:
@@ -472,7 +529,7 @@ def get_thumbnail(config: Configuration, video_id: str, video: Dict[str, Any]) -
     JPG and returns the filename of the converted file.
 
     :param config: Configuration class instance
-    :param video_id: Youtube video id
+    :param video_id: YouTube video id
     :param video: youtube-dl information dict about one particular video
 
     :return:  Filename of the converted Thumbnail
